]> git.sesse.net Git - movit/blobdiff - effect.h
Make the blur use the resize functionality, which also unbreaks the in-between sampli...
[movit] / effect.h
index f495867a42029128eaa7140747508fa76ebce696..c016a55070c2fb63585cd381c46c9e69a4de77a4 100644 (file)
--- a/effect.h
+++ b/effect.h
 #include <string>
 #include <vector>
 
-#include <GL/gl.h>
+#include <assert.h>
+
+#include "opengl.h"
+
+class EffectChain;
 
 // Can alias on a float[2].
 struct Point2D {
@@ -49,7 +53,7 @@ public:
        // the pixels, e.g. mirror, won't need to care, and can set this
        // to false. If so, the input gamma will be undefined.
        //
// Also see the note on needs_many_samples(), below.
        virtual bool needs_linear_light() const { return true; }
 
        // Whether this effect expects its input to be in the sRGB
@@ -58,22 +62,34 @@ public:
        // set of RGB primaries; you would currently not get YCbCr
        // or something similar).
        //
-       // Again, most effects will want this.
+       // Again, most effects will want this, but you can set it to false
+       // if you process each channel independently, equally _and_
+       // in a linear fashion.
        virtual bool needs_srgb_primaries() const { return true; }
 
// Whether this effect expects to be sampling many times from
// its input. If this is true, the framework will not chain the
        // input from other effects, but will store the results of the
        // chain to a temporary (RGBA fp16) texture and let this effect
        // sample directly from that.
        //
-       // Note that if you do _not_ set this, and do not sample on
-       // whole pixels (ie. you request linear filtering), it is undefined
-       // whether that filtering happen in linear gamma or not.
-       // It _could_ be (for instance in the case where the input is sRGB
-       // and your GPU applies gamma expansion before filtering), but you
-       // have no such guarantee. For most uses, however, this will be fine.
-       virtual bool needs_many_samples() const { return false; }
+       // There are two good reasons why you might want to set this:
+       //
+       //  1. You are sampling more than once from the input,
+       //     in which case computing all the previous steps might
+       //     be more expensive than going to a memory intermediate.
+       //  2. You rely on previous effects, possibly including gamma
+       //     expansion, to happen pre-filtering instead of post-filtering.
+       //     (This is only relevant if you actually need the filtering; if
+       //     you sample on whole input pixels only, it makes no difference.)
+       //
+       // Note that in some cases, you might get post-filtered gamma expansion
+       // even when setting this option. More specifically, if you are the
+       // first effect in the chain, and the GPU is doing sRGB gamma
+       // expansion, it is undefined (from OpenGL's side) whether expansion
+       // happens pre- or post-filtering. For most uses, however,
+       // either will be fine.
+       virtual bool needs_texture_bounce() const { return false; }
 
        // Whether this effect expects mipmaps or not. If you set this to
        // true, you will be sampling with bilinear filtering; if not,
@@ -82,13 +98,31 @@ public:
        // needs mipmaps, you will also get them).
        virtual bool needs_mipmaps() const { return false; }
 
+       // Whether this effect wants to output to a different size than
+       // its input(s). If you set this to true, the output will be
+       // bounced to a texture (similarly to if the next effect set
+       // needs_texture_bounce()).
+       virtual bool changes_output_size() const { return false; }
+
+       // If changes_output_size() is true, you must implement this to tell
+       // the framework what output size you want.
+       //
+       // Note that it is explicitly allowed to change width and height
+       // from frame to frame; EffectChain will reallocate textures as needed.
+       virtual void get_output_size(unsigned *width, unsigned *height) const {
+               assert(false);
+       }
+
+       // How many inputs this effect will take (a fixed number).
+       // If you have only one input, it will be called INPUT() in GLSL;
+       // if you have several, they will be INPUT1(), INPUT2(), and so on.
+       virtual unsigned num_inputs() const { return 1; }
+
        // Requests that this effect adds itself to the given effect chain.
        // For most effects, the default will be fine, but for effects that
        // consist of multiple passes, it is often useful to replace this
        // with something that adds completely different things to the chain.
-       virtual void add_self_to_effect_chain(std::vector<Effect *> *chain) {
-               chain->push_back(this);
-       }
+       virtual void add_self_to_effect_chain(EffectChain *graph, const std::vector<Effect *> &inputs);
 
        // Outputs one GLSL uniform declaration for each registered parameter
        // (see below), with the right prefix prepended to each uniform name.
@@ -98,17 +132,19 @@ public:
        // Returns the GLSL fragment shader string for this effect.
        virtual std::string output_fragment_shader() = 0;
 
// Set all uniforms the shader needs in the current GL context.
// The default implementation sets one uniform per registered parameter.
        //
        // <sampler_num> is the first free texture sampler. If you want to use
        // textures, you can bind a texture to GL_TEXTURE0 + <sampler_num>,
        // and then increment the number (so that the next effect in the chain
        // will use a different sampler).
-       //
-       // NOTE: Currently this is also abused a bit to set other GL state
-       // the effect might need.
-       virtual void set_uniforms(GLuint glsl_program_num, const std::string& prefix, unsigned *sampler_num);
+       virtual void set_gl_state(GLuint glsl_program_num, const std::string& prefix, unsigned *sampler_num);
+
+       // If you set any special OpenGL state in set_gl_state(), you can clear it
+       // after rendering here. The default implementation does nothing.
+       virtual void clear_gl_state();
 
        // Set a parameter; intended to be called from user code.
        // Neither of these take ownership of the pointer.
@@ -49,7 +53,7 @@ public:
        // to false. If so, the input gamma will be undefined.
        //
-       // Also see the note on needs_many_samples(), below.
+       // Also see the note on needs_texture_bounce(), below.
        virtual bool needs_linear_light() const { return true; }
 
@@ -58,22 +62,34 @@ public:
        virtual bool needs_srgb_primaries() const { return true; }
 
-       // Whether this effect expects to be sampling many times from
-       // its input. If this is true, the framework will not chain the
+       // Whether this effect expects its input to come directly from
+       // a texture. If this is true, the framework will not chain the
        // input from other effects, but will store the results of the
        // chain to a temporary (RGBA fp16) texture and let this effect
@@ -98,17 +132,19 @@ public:
        virtual std::string output_fragment_shader() = 0;
 
-       // Set all uniforms the shader needs in the current GL context.
-       // The default implementation sets one uniform per registered parameter.
+       // Set all OpenGL state that this effect needs before rendering.
+       // The default implementation sets one uniform per registered parameter,
+       // but no other state.
        //
        // <sampler_num> is the first free texture sampler. If you want to use